/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/o2kxml/. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2000-2002. All Rights Reserved. 

Contributor(s): 
*/

package com.simonstl.blogger;


import java.io.*;

/**
 * <p>This class converts XML documents which use a set of conventions into Blogger templates.</p>
 * 
 * <p>Because Blogger templates use tags like &lt;$BlogItemAuthor$&gt;, they aren't properly XML.  This code takes XHTML in the form &lt;_BlogItemAuthor_/&gt; and converts it to &lt;$BlogItemAuthor$&gt;.  You shouldn't run this code on other documents which have elements starting with _B or containing empty elements whose names end in _.  It also adds the necessary spaces to empty elements generated by XML processors that create things like &lt;br/&gt;.  If the spaces are already there, an extra space is inserted, but that causes no harm.</p>
 * 
 * 
 * @author Simon St.Laurent
 * @version 0.01 $Date: 2002/01/27 $
 */
public final class XML2blog extends FilterReader
{
    protected boolean pastXmlDecl=false;
    protected boolean generating=false;
    protected boolean inElementName=false;
    protected StringBuffer myBuffer = new StringBuffer();

    public XML2blog (InputStream in) {
      this(new InputStreamReader(in));
    }

    public XML2blog (Reader in) {
      super(new BufferedReader(in));
    }

/**
To keep the XML declaration, call keepXMLDecl(true) before processing the document.
**/

    public void keepXMLDecl(boolean _pastXmlDecl){
      pastXmlDecl=_pastXmlDecl;
    }

    protected int feedFromInternalBuffer() {
        //feed out the buffer a character at a time
        int retChar=myBuffer.charAt(0);
        myBuffer.reverse();
        myBuffer.setLength(myBuffer.length()-1);
        myBuffer.reverse();
        return retChar;
    }
    
    public int read() throws IOException {

        int c = ' '; //return a space if nothing else

      if (myBuffer.length()==0) {
       c=in.read();
       if (pastXmlDecl) { 
        if (c=='<') {
          myBuffer.append((char)c);
          int d=in.read();
          myBuffer.append((char)d);
          if (d=='_') {
            int e=in.read();
            myBuffer.append((char)e);
            if (e=='B') {
              myBuffer=new StringBuffer("<$B");
            }//end e;
          }//end d;
        } //end startElement (c)

        if (c=='_') {
          myBuffer.append((char)c);
          int d=in.read();
          myBuffer.append((char)d);
          if (d=='/') {
            int e=in.read();
            myBuffer.append((char)e);
            if (e=='>') {
              myBuffer=new StringBuffer("$>");
            }//end e;
          }//end d;
        } //end startElement (c)


        if (c=='/') {
          myBuffer.append((char)c);
          int d=in.read();
          myBuffer.append((char)d);
          if (d=='>') {
            myBuffer=new StringBuffer(" />");
          }
        } //end test for XHTML empty tags
       } else {
        if (c=='<') {
          pastXmlDecl=true;
          myBuffer.append((char)c);
          int d=in.read();
          myBuffer.append((char)d);
          if (d=='?') {
            int e=in.read();
            myBuffer.append((char)e);
            if (e=='x') {
              int f=in.read();
              myBuffer.append((char)f);
              if (f=='m') {
                int g=in.read();
                myBuffer.append((char)g);
                if (g=='l') {
                 int h=in.read();
                  myBuffer.append((char)h);
                  if ((h==' ') || (h==0x9) || (h==0xA) || (h==0xD)) {

                   //nuke everything to the closing > 
                   while (h!='>') {
                      h=in.read();
                   } //end while
                   c=in.read();
                   myBuffer=new StringBuffer();
                   myBuffer.append((char)c);
                 }//end h;
                }//end g
              }//end f;
            }//end e;
          }//end d;
        } //end startElement (c)

       }

      } //end buffer test

      if (myBuffer.length()>0) {
         c=feedFromInternalBuffer();
      } 
        return c;
    }

    /**
     * This method is a placeholder - all 'real' activity appears in 
     * the int read() method.  
     * This placeholder is substantially from Java I/O by 
     * Elliotte Rusty Harold, http://www.oreilly.com/catalog/javaio/.
     */
    private boolean endOfStream = false;
  
    public int read(char[] text, int offset, int length) throws IOException {
  
        if (endOfStream) return -1;
        int numRead = 0;
        
        for (int i = offset; i < offset+length; i++) {
            int temp = this.read();
            if (temp == -1) {
              this.endOfStream = true;
              break;
            }
            text[i] = (char) temp;
            numRead++;
        }
        return numRead;
  
    }

    /** This class defines a main() method to test the O2KCleaner */
    public static void main(String[] args) {
        
        if (args.length != 1) {
            System.out.println("Usage: java XML2blog <filename>");
            return;       
        }
        try {          
            // Create a stream to read and clean the file
            XML2blog blogFixer=new XML2blog(new FileReader(args[0]));
//            blogFixer.keepXMLDecl(true);
            BufferedReader in = new BufferedReader(blogFixer);
            String line;
            while((line = in.readLine()) != null) {
              System.out.println(line);
            }
            in.close();  // Close the stream.
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
    
}
